/***************************************************************
* Stream Device record interface for analog output records     *
*                                                              *
* (C) 1999 Dirk Zimoch (zimoch@delta.uni-dortmund.de)          *
* (C) 2005 Dirk Zimoch (dirk.zimoch@psi.ch)                    *
*                                                              *
* This is an EPICS record Interface for StreamDevice.          *
* Please refer to the HTML files in ../doc/ for a detailed     *
* documentation.                                               *
*                                                              *
* If you do any changes in this file, you are not allowed to   *
* redistribute it any more. If there is a bug or a missing     *
* feature, send me an email and/or your patch. If I accept     *
* your changes, they will go to the next release.              *
*                                                              *
* DISCLAIMER: If this software breaks something or harms       *
* someone, it's your problem.                                  *
*                                                              *
***************************************************************/

#include <aoRecord.h>
#include "devStream.h"
#include <epicsExport.h>

static long readData (dbCommon *record, format_t *format)
{
    aoRecord *ao = (aoRecord *) record;
    double val;

    switch (format->type)
    {
        case DBF_DOUBLE:
        {
            if (streamScanf (record, format, &val)) return ERROR;
            break;
        }
        case DBF_ULONG:
        case DBF_LONG:
        {
            long rval;
            if (streamScanf (record, format, &rval)) return ERROR;
            ao->rbv = rval;
            ao->rval = rval;
            if (ao->linr == 0)
            {
                /* allow integers with more than 32 bits */
                if (format->type == DBF_ULONG)
                    val = (unsigned long)rval;
                else    
                    val = rval;
                break;
            }
            return OK;
        }
        default:
            return ERROR;
    }
    if (ao->aslo != 0.0 && ao->aslo != 1.0) val *= ao->aslo;
    ao->val = val + ao->aoff;
    return DO_NOT_CONVERT;
}

static long writeData (dbCommon *record, format_t *format)
{
    aoRecord *ao = (aoRecord *) record;

    double val = (INIT_RUN ? ao->val : ao->oval) - ao->aoff;
    if (ao->aslo != 0.0 && ao->aslo != 1.0) val /= ao->aslo;
    
    switch (format->type)
    {
        case DBF_DOUBLE:
        {
            return streamPrintf (record, format, val);
        }
        case DBF_ULONG:
        {
            if (ao->linr == 0)
            {
                /* allow integers with more than 32 bits */
                return streamPrintf (record, format, (unsigned long)val);
            }
            return streamPrintf (record, format, (unsigned long)ao->rval);
        }
        case DBF_LONG:
        {
            if (ao->linr == 0)
            {
                /* allow integers with more than 32 bits */
                return streamPrintf (record, format, (long)val);
            }
            return streamPrintf (record, format, (long)ao->rval);
        }
    }
    return ERROR;
}

static long initRecord (dbCommon *record)
{
    aoRecord *ao = (aoRecord *) record;

    return streamInitRecord (record, &ao->out, readData, writeData);
}

struct {
    long number;
    DEVSUPFUN report;
    DEVSUPFUN init;
    DEVSUPFUN init_record;
    DEVSUPFUN get_ioint_info;
    DEVSUPFUN write;
    DEVSUPFUN special_linconv;
} devaoStream = {
    6,
    streamReport,
    streamInit,
    initRecord,
    streamGetIointInfo,
    streamWrite,
    NULL
};

epicsExportAddress(dset,devaoStream);
